package it.mokabyte.pattern.locator;

import java.util.Properties;

import java.io.*;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.EJBHome;

import javax.jms.TopicConnectionFactory;
import javax.jms.QueueConnectionFactory;

import it.mokabyte.pattern.util.SimpleTrace;
import it.mokabyte.pattern.util.JndiNames;
import it.mokabyte.pattern.exception.ServiceLocationException;

/**
 * <p>Title: MyServiceLocator</p>
 * <p>Description: this class provides Service Locator for EJB & JMS resources. Allows :</p>
 * <li> type checked location for locating EJB
 * <li> type unchecked location for locating EJB
 * <li< type unchecked locastion for locating JMS
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author S.Rossini
 * @version 1.0
 */
public class MyServiceLocator {

  /** Busienss Services ID for type checking */
  public final static int MOKA_ACCOUNT_SERVICE = 0;
  public final static int MOKA_SHOPPER_SERVICE = 1;
  public final static int MOKA_TOPIC_SERVICE   = 2;

  /** SUN RI Connection Factory class */
  private final static String SUN_RI_CF = "com.sun.jndi.cosnaming.CNCtxFactory";

  /** JBoss Connection Factory class */
  private final static String JBOSS_CF = "org.jnp.interfaces.NamingContextFactory";

  /** WebLogic Connection Factory class */
  private final static String WLS_CF = "weblogic.jndi.WLInitialContextFactory";

  /** Array containing ServiceConfig obj for type checked service locating
   *  <li> SLOT 0 : Account EJB from SUN RI
   *  <li> SLOT 1 : Shopper EJB from JBoss
   *  <li> SLOT 2 : JMS Topic from WLS
   */
  private static final Object businessServices[] = {

    /* 0. Account */ new ServiceConfig(it.mokabyte.pattern.ejb.entity.AccountHome.class,
                                       JndiNames.ACCOUNT,
                                       MyServiceLocator.SUN_RI_CF,
                                       "iiop://localhost:1050",
                                       null,
                                       null),
   /* 1. Shopper */ new ServiceConfig(it.mokabyte.pattern.ejb.session.stateful.ShopperHome.class,
                                       JndiNames.SHOPPER,
                                       MyServiceLocator.JBOSS_CF,
                                       "jnp://localhost:1099",
                                       null,
                                       null),
  /* 2. MokaTopic */ new ServiceConfig(it.mokabyte.pattern.ejb.session.stateful.ShopperHome.class,
                                       JndiNames.SHOPPER,
                                       MyServiceLocator.WLS_CF,
                                       "t3://localhost:7001",
                                       null,
                                       null),
   };


  /** singleton's private instance */
  private static MyServiceLocator me;

  // static { me = new MyServiceLocator();}

  private MyServiceLocator() {}

 /**
  * @return MyServiceLocator singleton instance
  */
  public static MyServiceLocator getInstance(){
    if (me == null) {
        synchronized(MyServiceLocator.class) {
            if (me == null) {
                me = new MyServiceLocator();
            }
        }
    }
    return me;
  }

  /**
    * Gets the EJBHome for the given service using the JNDI name and the Class for the EJBHome
    * corrispondig the type specified.
    * <li> From type received (param service) take the corresponding ServiceConfig object
    * <li> Create a JNDI Context and set by ServiceConfig
    * <li> lookup EJB
    * <li> perform the narrow
    * @param  service
    * @return EJBHome
    * @throws ServiceException
    */
  public EJBHome getHome( int service ) throws ServiceLocationException {

    EJBHome home = null;

    try {

      SimpleTrace.trace("MyServiceLocator.getHome : service ID["+service+"]...");

      if(service < 0 || service > businessServices.length){
          throw new ServiceLocationException(new Exception("MyServiceLocator.getHome : service ID OUT OF BOUNDS !"));
      }

      // Look up using the service name from defined constant
      ServiceConfig config = (ServiceConfig)MyServiceLocator.businessServices[service];

      SimpleTrace.trace("MyServiceLocator.getHome : ServiceConfig["+config+"]...");

      Context ctx = this.getInitialContext(config.getInitialContextFactory(),config.getProviderUrl());

      SimpleTrace.trace("MyServiceLocator.getHome : looking up for [" + config.getJndiName() + "]...");

      Object objref = ctx.lookup(config.getJndiName());

      SimpleTrace.trace("MyServiceLocator.getHome : objref class["+objref.getClass().getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getHome : objref ["+objref+"] ...");

      SimpleTrace.trace("MyServiceLocator.getHome : class to narrow : " + config.getHomeClass());

      // Narrow using the EJBHome Class from defined constant
      Object obj = PortableRemoteObject.narrow(objref,config.getHomeClass());

      home = (EJBHome)obj;

      SimpleTrace.trace("MyServiceLocator.getHome : home class["+home.getClass().getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getHome : home ["+home+"] ...");
    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( ClassCastException cce ) {
      cce.printStackTrace();
      throw new ServiceLocationException(cce);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(ex);
    }

    SimpleTrace.trace("MyServiceLocator.getHome : returning [" + home + "]...");
    return home;

   }

  /** gets the EJBHome for the given service using the given JNDI name and the EJBHome Class
    * <li> Create a JNDI Context (will take System properties set by java runtime flag -D, NOT ServiceConifg values)
    * <li> lookup EJB
    * <li> perform the narrow
    * @param  service
    * @return EJBHome the EJB factory
    * @throws ServiceException
    */
   public EJBHome getHome(String jndiName, Class theClass) throws ServiceLocationException {

    EJBHome home = null;

    try {

      Context ctx = new InitialContext();

      SimpleTrace.trace("MyServiceLocator.getHome : looking up for [" + jndiName + "]...");

      Object objref = ctx.lookup(jndiName);

      SimpleTrace.trace("MyServiceLocator.getHome : objref class["+theClass.getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getHome : objref ["+theClass+"] ...");

      // Narrow using the EJBHome Class from defined constant
      Object obj = PortableRemoteObject.narrow(objref,theClass);

      home = (EJBHome)obj;

      SimpleTrace.trace("MyServiceLocator.getHome : home class["+home.getClass().getName()+"] ...");
      SimpleTrace.trace("MyServiceLocator.getHome : home ["+home+"] ...");

    }
    catch( NamingException ne ) {
      ne.printStackTrace();
      throw new ServiceLocationException(ne);
    }
    catch( Exception ex ) {
      ex.printStackTrace();
      throw new ServiceLocationException(new Exception(ex.getMessage()));
    }

    System.out.println("MyServiceLocator.getHome : returning [" + home + "]...");
    return home;

  }

  /**
   * i.e: type un-checked JMS
   * @param CxtFact the ContextFactory class name
   * @param provUrl the provider URL
   * @param jndiName the JNDI name
   * @return the Topic JMS Factory
   * @throws ServiceLocationException
   */
  public TopicConnectionFactory getTopicHome(String jndiTopicConnectionFactoryName) throws ServiceLocationException {

          try {

            Context ctx = new InitialContext();
            SimpleTrace.trace("JNDI Context["+ctx+"]...");

            // Look up connection ConnectionFactory e detination Topic
            SimpleTrace.trace("Looking up connection factory ["+ jndiTopicConnectionFactoryName+"]...");
            TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory)ctx.lookup(jndiTopicConnectionFactoryName);

            SimpleTrace.trace("topicConnectionFactory [" + topicConnectionFactory + "]...");
            SimpleTrace.trace("topicConnectionFactory class [" + topicConnectionFactory.getClass().getName() + "]...");

            return topicConnectionFactory;

        } catch (NamingException ne) {
            SimpleTrace.trace("JNDI lookup failed : " + ne.toString());
            ne.printStackTrace();
            throw new ServiceLocationException(ne);
        }
  }

 /**
   * type un-checked JMS
   * @param CxtFact the ContextFactory class name
   * @param provUrl the provider URL
   * @param jndiName the JNDI name
   * @return the Queue JMS Factory
   * @throws ServiceLocationException
   */
  public QueueConnectionFactory getQueueHome(String jndiQueueConnectionFactoryName) throws ServiceLocationException {

          try {

            Context ctx = new InitialContext();
            SimpleTrace.trace("JNDI Context["+ctx+"]...");

            // Look up connection ConnectionFactory e detination Topic
            SimpleTrace.trace("Looking up connection factory ["+ jndiQueueConnectionFactoryName+"]...");
            QueueConnectionFactory queueConnectionFactory = (QueueConnectionFactory)ctx.lookup(jndiQueueConnectionFactoryName);

            SimpleTrace.trace("queueConnectionFactory [" + queueConnectionFactory + "]...");
            SimpleTrace.trace("queueConnectionFactory class [" + queueConnectionFactory.getClass().getName() + "]...");

            return queueConnectionFactory;

        } catch (NamingException ne) {
            SimpleTrace.trace("JNDI lookup failed : " + ne.toString());
            ne.printStackTrace();
            throw new ServiceLocationException(ne);
        }
  }

  /**
   * get the initial context with INITIAL_CONTEXT_FACTORY and PROVIDER_URL specified
   * @param ctxFacxtory
   * @param provUrl
   * @return default context
   * @throws NamingException
   */
  private Context getInitialContext(String ctxFacxtory, String provUrl) throws NamingException {

      Properties properties = null;

      properties = new Properties();
      properties.put(Context.INITIAL_CONTEXT_FACTORY, ctxFacxtory);
      properties.put(Context.PROVIDER_URL, provUrl);

      Context ctx = new InitialContext(properties);
      this.printContextProperties(ctx);
      return ctx;
     }

  /**
   * get the initial context with also security credentials
   * @param ctxFacxtory
   * @param provUrl
   * @return default context
   * @throws NamingException
   */
  private Context getInitialContext(String ctxFacxtory, String provUrl,String username, String password) throws NamingException {

      Properties properties = null;

      properties = new Properties();
      properties.put(Context.INITIAL_CONTEXT_FACTORY, ctxFacxtory);
      properties.put(Context.PROVIDER_URL, provUrl);
      properties.put(Context.SECURITY_PRINCIPAL, username);
      properties.put(Context.SECURITY_CREDENTIALS, password == null ? "" : password);

      Context ctx = new InitialContext(properties);
      this.printContextProperties(ctx);
      return ctx;
     }

  /**
   * simple print ofthe context properties
   * @param jndiContext
   * @throws NamingException
   */
  private static void printContextProperties(Context jndiContext) throws NamingException {
     SimpleTrace.trace("ContextHelper.printContextProperties : Context initialized ["+jndiContext+"]...");

     java.util.Hashtable ht = jndiContext.getEnvironment();
     java.util.Enumeration enum = ht.keys();
     while(enum.hasMoreElements()) {
          String str = (String)enum.nextElement();
          SimpleTrace.trace("MyServiceLocator.getHome : CTX Prop["+str + "] - value[" + ht.get(str)+"]");
    }
  }
}

/**
 * <p>Title: ServiceConfig</p>
 * <p>Description: services configuration utility class</p>
 */
class ServiceConfig {

    private Class homeClass = null;
    private String jndiName = null;
    private String initialContextFactory = null;
    private String providerUrl = null;
    private String username = null;
    private String password = null;

    ServiceConfig(Class h, String jn, String icf, String pu, String u, String p) {
        this.homeClass = h;
        this.jndiName = jn;
        this.initialContextFactory = icf;
        this.providerUrl = pu;
        this.username = u;
        this.password = p;
    }

    Class getHomeClass() {
        return this.homeClass;
    }

    String getJndiName() {
        return this.jndiName;
    }

    String getInitialContextFactory() {
        return this.initialContextFactory;
    }

    String getProviderUrl() {
        return this.providerUrl;
    }

    String getUserName() {
        return this.username;
    }

    String getPassword() {
        return this.password;
    }

    public String toString() {
        return "MyServiceLocator : home[" + this.homeClass + "]-CTX Factory["+this.initialContextFactory+"]-JNDI Name["+this.jndiName+"]-URL["+this.providerUrl+"]-UserName["+this.username+"]-Password["+this.password+"]";
    }
}


